package rabbit.zip;

import java.util.zip.DataFormatException;
import java.util.zip.Inflater;

/** GZipState for uncompressing the 'deflate' data 
*
* @author Rick Leir
*/
public class DeCompressor implements GZipUnpackState {

    private GZipUnpackListener listener;
    private Inflater decompresser;

    public DeCompressor (GZipUnpackListener listener) {
	this.listener = listener;
	decompresser = new Inflater (true);
    }

    public boolean needsInput () {
	return decompresser.needsInput ();
    }

    public void handleBuffer (GZipUnpacker unpacker, 
			      byte[] buf, int off, int len) {
	decompresser.setInput (buf, off, len);
	handleCurrentData (unpacker);
    }

    public void handleCurrentData (GZipUnpacker unpacker) {
	byte[] unpacked = listener.getBuffer ();
	int num = -1;
	try {
	    // Decompress the bytes
	    if ((num = decompresser.inflate (unpacked)) > 0) {
		listener.unpacked (unpacked, 0, num);
	    }
	} catch (DataFormatException e) {
	    listener.failed (e); // where does this go?  test
	    return;
	}
	
	if (num == 0) {
	    // this path is never followed 
	    // ChunkHandler 56 need to call handleChunkData a last time to 
	    // wrap up or listener.finishedRead (); /rick
	    if (decompresser.finished () || decompresser.needsDictionary ()) {
		decompresser.end ();
		unpacker.setState (new AfterEndState ());
		listener.finished ();
	    }
	}
    }
}
